@@ -13,6 +13,9 @@ setClass("TextDocument",
                         LocalMetaData = "list",
                         "VIRTUAL"))
 
+# Minimal document
+#setOldClass("MinimalDocument")
+
 setGeneric("Author", function(object) standardGeneric("Author"))
 setMethod("Author", "TextDocument", function(object) object@Author)
 setGeneric("Author<-", function(x, value) standardGeneric("Author<-"))
@@ -39,6 +42,7 @@ setReplaceMethod("Description", "TextDocument", function(x, value) {
 
 setGeneric("ID", function(object) standardGeneric("ID"))
 setMethod("ID", "TextDocument", function(object) object@ID)
+#setMethod("ID", "MinimalDocument", function(object) attr(object, "ID"))
 setGeneric("ID<-", function(x, value) standardGeneric("ID<-"))
 setReplaceMethod("ID", "TextDocument", function(x, value) {
   x@ID <- value
@@ -63,6 +67,7 @@ setReplaceMethod("Heading", "TextDocument", function(x, value) {
 
 setGeneric("Language", function(object) standardGeneric("Language"))
 setMethod("Language", "TextDocument", function(object) object@Language)
+#setMethod("Language", "MinimalDocument", function(object) attr(object, "Language"))
 setGeneric("Language<-", function(x, value) standardGeneric("Language<-"))
 setReplaceMethod("Language", "TextDocument", function(x, value) {
   x@Language <- value
@@ -74,38 +79,27 @@ setMethod("LocalMetaData", "TextDocument", function(object) object@LocalMetaData
 
 # Derived text documents
 
-# Define class CallOrNULL as union of call and NULL
-setClassUnion("callOrNULL", c("call", "NULL"))
-
 # Plain text documents
 setClass("PlainTextDocument",
-         representation(URI = c("callOrNULL"), Cached = "logical"),
          contains = c("character", "TextDocument"))
 
 setGeneric("Content", function(object) standardGeneric("Content"))
 setMethod("Content", "PlainTextDocument", function(object) object@.Data)
+#setMethod("Content", "MinimalDocument", function(object) as.character(object))
 setGeneric("Content<-", function(x, value) standardGeneric("Content<-"))
+#setReplaceMethod("Content", "MinimalDocument", function(x, value) {
+#    attributes(value) <- attributes(x)
+#    value
+#})
 setReplaceMethod("Content", "PlainTextDocument", function(x, value) {
-  x@.Data <- value
-  x
-})
-
-setGeneric("URI", function(object) standardGeneric("URI"))
-setMethod("URI", "PlainTextDocument", function(object) object@URI)
-
-setGeneric("Cached", function(object) standardGeneric("Cached"))
-setMethod("Cached", "PlainTextDocument", function(object) object@Cached)
-setGeneric("Cached<-", function(x, value) standardGeneric("Cached<-"))
-setReplaceMethod("Cached", "PlainTextDocument", function(x, value) {
-  x@Cached <- value
-  x
+    x@.Data <- value
+    x
 })
 
 # XML text document
 # If XMLDocument would be a S4 class, we could directly inherit from it
 # Instead we have to do a work-around with a list
 setClass("XMLTextDocument",
-         representation(URI = "callOrNULL", Cached = "logical"),
          contains = c("list", "TextDocument"))
 
 setMethod("Content", "XMLTextDocument", function(object) object@.Data)
@@ -113,12 +107,6 @@ setReplaceMethod("Content", "XMLTextDocument", function(x, value) {
     x@.Data <- value
     x
 })
-setMethod("URI", "XMLTextDocument", function(object) object@URI)
-setMethod("Cached", "XMLTextDocument", function(object) object@Cached)
-setReplaceMethod("Cached", "XMLTextDocument", function(x, value) {
-    x@Cached <- value
-    x
-})
 
 # Reuters21578 XML document
 setClass("Reuters21578Document",
@@ -130,7 +118,7 @@ setClass("RCV1Document",
 
 # Newsgroup document as found in the Newsgroup dataset of the UCI KDD archive
 setClass("NewsgroupDocument",
-         representation(Newsgroup = "character", URI = "callOrNULL", Cached = "logical"),
+         representation(Newsgroup = "character"),
          contains = c("character", "TextDocument"))
 
 setMethod("Content", "NewsgroupDocument", function(object) object@.Data)
@@ -138,16 +126,9 @@ setReplaceMethod("Content", "NewsgroupDocument", function(x, value) {
     x@.Data <- value
     x
 })
-setMethod("URI", "NewsgroupDocument", function(object) object@URI)
-setMethod("Cached", "NewsgroupDocument", function(object) object@Cached)
-setReplaceMethod("Cached", "NewsgroupDocument", function(x, value) {
-  x@Cached <- value
-  x
-})
 
 # Structured text document for sectioned or structured text corpora
 setClass("StructuredTextDocument",
-         representation(URI = "callOrNULL", Cached = "logical"),
          contains = c("list", "TextDocument"))
 
 setMethod("Content", "StructuredTextDocument", function(object) object@.Data)
@@ -155,12 +136,6 @@ setReplaceMethod("Content", "StructuredTextDocument", function(x, value) {
     x@.Data <- value
     x
 })
-setMethod("URI", "StructuredTextDocument", function(object) object@URI)
-setMethod("Cached", "StructuredTextDocument", function(object) object@Cached)
-setReplaceMethod("Cached", "StructuredTextDocument", function(x, value) {
-    x@Cached <- value
-    x
-})
 
 # A node in the metadata tree of a corpus
 setClass("MetaDataNode",
@@ -169,38 +144,41 @@ setClass("MetaDataNode",
                         children = "list"))
 
 # Corpus (= text document collection)
-setClass("Corpus",
-         representation(DMetaData = "data.frame", CMetaData = "MetaDataNode", DBControl = "list"),
-         contains = c("list"))
+setClass("Corpus", representation(DMetaData = "data.frame", CMetaData = "MetaDataNode", "VIRTUAL"))
+
+# Standard corpus
+setClass("VCorpus", contains = c("list", "Corpus"))
+
+# Fast corpus
+setClass("FCorpus", contains = c("list", "Corpus"))
+
+# Permanent corpus
+setClass("PCorpus", representation(DBControl = "list"), contains = c("list", "Corpus"))
 
 # DMetaData = *MetaData* available for all *D*ocuments
 setGeneric("DMetaData", function(object) standardGeneric("DMetaData"))
-setMethod("DMetaData", "Corpus",
+#setMethod("DMetaData", "FCorpus", function(object) object@DMetaData)
+setMethod("DMetaData", "VCorpus", function(object) object@DMetaData)
+setMethod("DMetaData", "PCorpus",
           function(object) {
-              if (DBControl(object)[["useDb"]] && require("filehash")) {
-                  db <- dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
-                  result <- dbFetch(db, "DMetaData")
-                  index <- object@DMetaData[[1, "subset"]]
-                  if (!any(is.na(index)))
-                      result <- result[index, , drop = FALSE]
-                  return(result)
-              }
-              else
-                  return(object@DMetaData)
+              db <- filehash::dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
+              result <- filehash::dbFetch(db, "DMetaData")
+              index <- object@DMetaData[[1, "subset"]]
+              if (!any(is.na(index)))
+                  result <- result[index, , drop = FALSE]
+              result
           })
 setGeneric("DMetaData<-", function(x, value) standardGeneric("DMetaData<-"))
-setReplaceMethod("DMetaData", "Corpus",
+setReplaceMethod("DMetaData", "VCorpus", function(x, value) {
+  x@DMetaData <- value
+  x
+})
+setReplaceMethod("DMetaData", "PCorpus",
                  function(x, value) {
-                     if (DBControl(x)[["useDb"]] && require("filehash")) {
-                         db <- dbInit(DBControl(x)[["dbName"]], DBControl(x)[["dbType"]])
-                         db[["DMetaData"]] <- value
-                         x@DMetaData[[1, "subset"]] <- NA
-                         return(x)
-                     }
-                     else {
-                         x@DMetaData <- value
-                         return(x)
-                     }
+                     db <- filehash::dbInit(DBControl(x)[["dbName"]], DBControl(x)[["dbType"]])
+                     db[["DMetaData"]] <- value
+                     x@DMetaData[[1, "subset"]] <- NA
+                     x
                  })
 
 # CMetaData = *MetaData* describing only the Document *C*ollection itself
@@ -208,7 +186,7 @@ setGeneric("CMetaData", function(object) standardGeneric("CMetaData"))
 setMethod("CMetaData", "Corpus", function(object) object@CMetaData)
 
 setGeneric("DBControl", function(object) standardGeneric("DBControl"))
-setMethod("DBControl", "Corpus", function(object) object@DBControl)
+setMethod("DBControl", "PCorpus", function(object) object@DBControl)
 
 # Repository for corpora
 setClass("TextRepository",
