@@ -21,7 +21,7 @@ weightTfIdf <-
     WeightFunction(function(m) {
         isDTM <- inherits(m, "DocumentTermMatrix")
         if (isDTM) m <- t(m)
-        m <- .tfidf(m)
+        m <- m * log2(nDocs(m) / rowSums(m > 0))
         if (isDTM) t(m) else m
     }, "term frequency - inverse document frequency", "tf-idf")
 
@@ -30,30 +30,3 @@ weightBin <-
         m$v <- rep(1, length(m$v))
         m
     }, "binary", "bin")
-
-.weight_rows_of_simple_triplet_matrix <-
-function(x, w)
-{
-    x$v <- x$v * w[x$i]
-    x
-}
-
-.row_sums_of_simple_triplet_matrix <-
-function(x)
-{
-    ## Could add na.rm argument ...
-    y <- double(x$nrow)
-    sums <- tapply(x$v, x$i, sum)
-    y[as.numeric(names(sums))] <- sums
-    y
-}
-
-.tfidf <-
-function(m)
-{
-    ## m a simple_triplet_matrix.
-    ## Currently we cannot do m > 0, hence:
-    m$v <- m$v > 0
-    s <- .row_sums_of_simple_triplet_matrix(m)
-    .weight_rows_of_simple_triplet_matrix(m, log2(m$ncol / s))
-}
